@@ -5,9 +5,9 @@
 # Author: James Henstridge <james@daa.com.au>
 #
 # TODO
-# 1. fix up GtkDrawingArea, GtkImage and GtkPixmap
-# 2. document some of this (maybe all of it)
-# 3. add wrappers for some of the GDK types.
+# 1. document some of this (maybe all of it)
+# 2. add wrappers for some of the GDK types.
+# 3. fix up GtkImage
 
 # The _obj argument in each constructor is used to create a Gtkinter object
 # that coresponds to an already created GTK Object.  It should be possible
@@ -122,6 +122,8 @@ class GtkAdjustment(GtkData):
 		raise AttributeError, attr
 	def set_value(self, value):
 		_gtk.gtk_adjustment_set_value(self._o, value)
+	def clamp_page(self, lower, upper):
+		_gtk.gtk_adjustment_clamp_page(self._o, lower, upper)
 
 class GtkTooltips(GtkData):
 	def __init__(self, _obj=None):
@@ -178,6 +180,8 @@ class GtkWidget(GtkObject):
 		return _gtk.gtk_widget_get_style(self._o)
 	def get_toplevel(self):
 		return _obj2inst(_gtk.gtk_widget_get_toplevel(self._o))
+	def get_window(self):
+		return _gtk.gtk_widget_get_window(self._o)
 	def grab_focus(self, obj=None):
 		_gtk.gtk_widget_grab_focus(self._o)
 	def grab_default(self, obj=None):
@@ -234,7 +238,9 @@ class GtkWidget(GtkObject):
 	def set_usize(self, w, h):
 		_gtk.gtk_widget_set_usize(self._o, w, h)
 	def shape_combine_mask(self, mask, ofs_x, ofs_y):
-	        _gtk.gtk_widget_shape_combine_mask(self._o,mask._o,ofs_x,ofs_y)
+		if hasattr(mask, '_o'):
+			tmp, mask = mask.get()
+	        _gtk.gtk_widget_shape_combine_mask(self._o, mask, ofs_x, ofs_y)
 	def show(self, obj=None):
 		_gtk.gtk_widget_show(self._o)
 	def show_all(self, obj=None):
@@ -279,6 +285,12 @@ class GtkContainer(GtkWidget):
 		_gtk.gtk_container_remove(self._o, child._o)
 	def unblock_resize(self, obj=None):
 		_gtk.gtk_container_unblock_resize(self._o)
+	def set_focus_child(self, child):
+		_gtk.gtk_container_set_focus_child(self._o, child._o)
+	def set_focus_vadjustment(self, adj):
+		_gtk.gtk_container_set_focus_vadjustment(self._o, adj._o)
+	def set_focus_hadjustment(self, adj):
+		_gtk.gtk_container_set_focus_hadjustment(self._o, adj._o)
 
 class GtkBin(GtkContainer):
 	def __init__(self, _obj=None):
@@ -544,8 +556,10 @@ class GtkWindow(GtkBin):
 		_gtk.gtk_window_position(self._o, pos)
 	def set_wmclass(self, wmc_name, wmc_class):
 		_gtk.gtk_window_set_wmclass(self._o, wmc_name, wmc_class)
-	def set_icon(self, pixmap):
-		_gtk.gtk_window_set_icon(self._o, pixmap._o)
+	def set_icon(self, pixmap, mask=None):
+		if not mask and hasattr(pixmap, '_o'):
+			pixmap, mask = pixmap.get()
+		_gtk.gtk_window_set_icon(self._o, pixmap, mask)
 	def set_icon_name(self, name):
 		_gtk.gtk_window_set_icon_name(self._o, name)
 
@@ -774,6 +788,10 @@ class GtkCList(GtkContainer):
 		return _gtk.gtk_clist_get_row_data(self._o, row)
 	def get_text(self, r, c):
 		return _gtk.gtk_clist_get_text(self._o, r, c)
+	def get_pixmap(self, r, c):
+		return _gtk.gtk_clist_get_pixmap(self._o, r, c)
+	def get_pixtext(self, r, c):
+		return _gtk.gtk_clist_get_pixtext(self._o, r, c)
 	def insert(self, row, values):
 		_gtk.gtk_clist_insert(self._o, row, values)
 	def moveto(self, row=-1, col=-1, row_align=0.5, col_align=0.5):
@@ -808,10 +826,15 @@ class GtkCList(GtkContainer):
 		_gtk.gtk_clist_set_selection_mode(self._o, mode)
 	def set_text(self, r, c, text):
 		_gtk.gtk_clist_set_text(self._o, r, c, text)
-	def set_pixmap(self, r, c, pixmap):
-		_gtk.gtk_clist_set_pixmap(self._o, r, c, pixmap._o)
-	def set_pixtext(self, r, c, text, spacing, pixmap):
-		_gtk.gtk_clist_set_pixtext(self._o,r,c,text,spacing,pixmap._o)
+	def set_pixmap(self, r, c, pixmap, mask=None):
+		if not mask and hasattr(pixmap, '_o'):
+			pixmap, mask = pixmap.get()
+		_gtk.gtk_clist_set_pixmap(self._o, r, c, pixmap, mask)
+	def set_pixtext(self, r, c, text, spacing, pixmap, mask=None):
+		if not mask and hasattr(pixmap, '_o'):
+			pixmap, mask = pixmap.get()
+		_gtk.gtk_clist_set_pixtext(self._o, r, c, text, spacing,
+					   pixmap, mask)
 	def thaw(self):
 		_gtk.gtk_clist_thaw(self._o)
 	def unselect_row(self, row, col):
@@ -1083,10 +1106,50 @@ class GtkToolbar(GtkContainer):
 
 class GtkDrawingArea(GtkWidget):
 	def __init__(self, _obj=None):
+		self.__win = None
 		if _obj: self._o = _obj; return
 		self._o = _gtk.gtk_drawing_area_new()
 	def size(self, w, h):
 		_gtk.gtk_drawing_area_size(self._o, w, h)
+	def __cache_win(self):
+		self.realize()
+		self.__win = self.get_window()
+	def draw_point(self, gc, x, y):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_point(self.__win, gc, x, y)
+	def draw_line(self, gc, x1, y1, x2, y2):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_line(self.__win, gc, x1, y1, x2, y2)
+	def draw_rectangle(self, gc, fill, x, y, width, height):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_rectangle(self.__win, gc, fill, x, y,
+					width, height)
+	def draw_arc(self, gc, x, y, width, height, angle1, angle2):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_arc(self.__win, gc, x, y, width, height, angle1,
+				  angle2)
+	def draw_polygon(self, gc, points):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_polygon(self.__win, gc, points)
+	def draw_string(self, gc, x, y, string):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_string(self.__win, gc, x, y, string)
+	def draw_text(self, gc, x, y, text):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_text(self.__win, gc, x, y, text)
+	def draw_pixmap(self, gc, src, xsrc,ysrc, xdest,ydest, width,height):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_pixmap(self.__win, gc, src, xsrc,ysrc,
+				     xdest,ydest, width,height)
+	def draw_points(self, gc, points):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_points(self.__win, gc, points)
+	def draw_segments(self, gc, segs):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_segments(self.__win, gc, segs)
+	def draw_lines(self, gc, points):
+		if not self.__win: self.__cache_win()
+		_gtk.gdk_draw_lines(self.__win, gc, points)
 
 class GtkCurve(GtkDrawingArea):
 	def __init__(self, _obj=None):
@@ -1244,12 +1307,25 @@ class GtkImage(GtkMisc):
 	# XXX should have __init__, set, get
 
 class GtkPixmap(GtkMisc):
-	def __init__(self, parent=None, xpm=None, bg=None, _obj=None):
+	def __init__(self, parent_or_pixmap=None, xpm_or_mask=None, bg=None,
+		     _obj=None):
+		# This function can be called with two argument models:
+		# The old model:
+		#  GtkPixmap(parent_win, xpm_file, bg_colour)
+		# or the one closer to the C version:
+		#  GtkPixmap(pixmap, mask)
 		if _obj: self._o = _obj; return
-		if bg:
-			self._o =_gtk.gtk_pixmap_new_from_xpm(parent._o,xpm,bg)
+		if type(xpm_or_mask) == type(''):
+			pix, mask = create_pixmap_from_xpm(parent_or_pixmap,
+							   bg, xpm_or_mask)
+			self._o = _gtk.gtk_pixmap_new(pix, mask)
 		else:
-			self._o = _gtk.gtk_pixmap_new_from_xpm(parent._o, xpm)
+			self._o = _gtk.gtk_pixmap_new(parent_or_pixmap,
+						      xpm_or_mask)
+	def get(self):
+		return _gtk.gtk_pixmap_get(self._o)
+	def set(self, pixmap, mask):
+		_gtk.gtk_pixmap_set(self._o, pixmap, mask)
 
 class GtkLabel(GtkMisc):
 	def __init__(self, label="", _obj=None):
@@ -1533,3 +1609,43 @@ def colour_alloc(parent, red, green=None, blue=None):
 	return _gtk.gdk_color_alloc(parent, red, green, blue)
 # This one's for the Americans...
 color_alloc = colour_alloc
+
+# pixmap loading
+def create_pixmap_from_xpm(window, bg, xpm):
+	if hasattr(window, '_o'):
+		window.realize()
+		window = window.get_window()
+	return _gtk.gdk_pixmap_create_from_xpm(window, bg, xpm)
+def create_pixmap(window, width, height, depth=-1):
+	if (hasattr(window, '_o')):
+	        window.realize()
+		window = window.get_window()
+	return _gtk.gdk_pixmap_new(window, width, height, depth)
+
+# drawing functions...
+def draw_point(drawable, gc, x, y):
+	_gtk.gdk_draw_point(drawable, gc, x, y)
+def draw_line(drawable, gc, x1, y1, x2, y2):
+	_gtk.gdk_draw_line(drawable, gc, x1, y1, x2, y2)
+def draw_rectangle(drawable, gc, fill, x, y, width, height):
+	_gtk.gdk_draw_rectangle(drawable, gc, fill, x, y,
+				width, height)
+def draw_arc(drawable, gc, x, y, width, height, angle1, angle2):
+	_gtk.gdk_draw_arc(drawable, gc, x, y, width, height, angle1,
+			  angle2)
+def draw_polygon(drawable, gc, points):
+	_gtk.gdk_draw_polygon(drawable, gc, points)
+def draw_string(drawable, gc, x, y, string):
+	_gtk.gdk_draw_string(drawable, gc, x, y, string)
+def draw_text(drawable, gc, x, y, text):
+	_gtk.gdk_draw_text(drawable, gc, x, y, text)
+def draw_pixmap(drawable, gc, src, xsrc,ysrc, xdest,ydest, width,height):
+	_gtk.gdk_draw_pixmap(drawable, gc, src, xsrc,ysrc,
+			     xdest,ydest, width,height)
+def draw_points(drawable, gc, points):
+	_gtk.gdk_draw_points(drawable, gc, points)
+def draw_segments(drawable, gc, segs):
+	_gtk.gdk_draw_segments(drawable, gc, segs)
+def draw_lines(drawable, gc, points):
+	_gtk.gdk_draw_lines(drawable, gc, points)
+
