@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/env python
 
 from Gtkinter import *
 import GtkExtra
@@ -227,6 +227,8 @@ def make_toolbar(win):
 		if not win.flags(REALIZED):
 			win.realize()
 
+		pix, mask = create_pixmap_from_xpm(win, None, "test.xpm")
+
 		def set_toolbar_horizontal(_b, toolbar=toolbar):
 			toolbar.set_orientation(ORIENTATION_HORIZONTAL)
 		def set_toolbar_vertical(_b, toolbar=toolbar):
@@ -249,24 +251,24 @@ def make_toolbar(win):
 
 		toolbar.append_item("Horizontal", "Horizontal toolbar layout",
 				    "Horizontal toolbar layout",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_horizontal)
 		toolbar.append_item("Vertical", "Vertical toolbar layout",
 				    "Vertical toolbar layout",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_vertical)
 		toolbar.append_space()
 		toolbar.append_item("Icons", "Only show toolbar icons",
 				    "Only Icons",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_icons)
 		toolbar.append_item("Text", "Only show toolbar text",
 				    "Only Text",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_text)
 		toolbar.append_item("Both", "Show toolbar icons and text",
 				    "Both Text and Icons",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_both)
 		toolbar.append_space()
 
@@ -277,20 +279,20 @@ def make_toolbar(win):
 		toolbar.append_space()
 		toolbar.append_item("Small", "Use small spaces",
 				    "Small spaces",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_small_space)
 		toolbar.append_item("Big", "Use big spaces",
 				    "Big spaces",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_big_space)
 		toolbar.append_space()
 		toolbar.append_item("Enable", "Enable tooltips",
 				    "Enable tooltips",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_enable)
 		toolbar.append_item("Disable", "Disable tooltips",
 				    "Disable tooltips",
-				    GtkPixmap(win, "test.xpm", bg),
+				    GtkPixmap(pix, mask),
 				    set_toolbar_disable)
 		return toolbar
 
